@@ -6,7 +6,7 @@
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) version 3, or any
     later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), Nokia Corporation 
+    successor approved by the membership of KDE e.V.), Nokia Corporation
     (or its successors, if any) and the KDE Free Qt Foundation, which shall
     act as a proxy defined in Section 6 of version 3 of the license.
 
@@ -15,14 +15,14 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public 
+    You should have received a copy of the GNU Lesser General Public
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 #ifndef PHONON_EXPERIMENTAL_AVCAPTURE_H
 #define PHONON_EXPERIMENTAL_AVCAPTURE_H
 
-#if defined(QT_NO_PHONON_VIDEOCAPTURE) || defined(QT_NO_PHONON_AUDIOCAPTURE)
+#if defined(PHONON_NO_VIDEOCAPTURE) || defined(PHONON_NO_AUDIOCAPTURE)
 #define NO_PHONON_AVCAPTURE
 #endif
 
@@ -32,7 +32,12 @@
 #include "../medianode.h"
 #include "../phonondefs.h"
 #include "objectdescription.h"
+
+#if defined(MAKE_PHONONEXPERIMENTAL_LIB)
 #include "../phononnamespace.h"
+#else
+#include "phonon/phononnamespace.h"
+#endif
 
 class QString;
 class QStringList;
@@ -50,9 +55,8 @@ namespace Experimental
      *
      * This class gives you access to the capture capabilities of the backend.
      * There might be more than only one possible capture source, for audio, as
-     * well as for video. The list of available sources is available through
-     * BackendCapabilities::availableAudioCaptureDevices and
-     * BackendCapabilities::availableVideoCaptureDevices.
+     * well as for video. A prefferable device may easily be obtained by providing
+     * a capture category.
      *
      * @ingroup Recording
      * @author Matthias Kretz <kretz@kde.org>
@@ -68,54 +72,88 @@ namespace Experimental
         Q_PROPERTY(Phonon::VideoCaptureDevice videoCaptureDevice READ videoCaptureDevice WRITE setVideoCaptureDevice)
         public:
             /**
-             * Returns the currently used capture source for the audio signal.
+             * Constructs an AvCapture with the devices preferred for the specified
+             * capture category.
              *
-             * @see AudioCaptureDevice
-             * @see setAudioCaptureDevice(const AudioCaptureDevice &)
-             * @see setAudioCaptureDevice(int)
+             * @param category Used to determine what devices are most suited for
+             * the AvCapture.
+             *
+             * @see CaptureCategory
+             */
+            AvCapture(Phonon::CaptureCategory category, QObject *parent = NULL);
+
+            /**
+             * Returns the current state of the capture.
+             *
+             * @li If only the audio capture device is valid, it returns the audio capture state.
+             * @li If only the video capture device is valid, it returns the video capture state.
+             * @li If both the audio and video capture devices are valid, it only returns the
+             * video capture state.
+             */
+            State state() const;
+
+            /**
+             * Returns the currently used capture source for the audio signal.
              */
             Phonon::AudioCaptureDevice audioCaptureDevice() const;
 
             /**
              * Returns the currently used capture source for the video signal.
-             *
-             * @see VideoCaptureDevice
-             * @see setVideoCaptureDevice(const VideoCaptureDevice &)
-             * @see setVideoCaptureDevice(int)
              */
             Phonon::VideoCaptureDevice videoCaptureDevice() const;
 
+            /**
+             * Sets both the video and audio devices to the ones most suited for the
+             * specified category
+             *
+             * @param category Used to determine what devices are most suited for
+             * the AvCapture.
+             */
+            void setCaptureDevices(Phonon::CaptureCategory category);
+
             /**
              * Sets the audio capture source to use.
              *
-             * @param source An object of class AudioCaptureDevice. A list of
-             * available objects can be queried from
-             * BackendCapabilities::availableAudioCaptureDevices.
+             * @param source An object of class AudioCaptureDevice.
              *
-             * @see audioCaptureDevice
-             * @see setAudioCaptureDevice(int)
              */
             void setAudioCaptureDevice(const Phonon::AudioCaptureDevice &source);
-            void setAudioCaptureDevice(Phonon::Category category);
+
+            /**
+             * Sets the audio capture device by using a capture category to get
+             * the appropriate device.
+             *
+             * @param category Capture category to use for getting a device
+             */
+            void setAudioCaptureDevice(Phonon::CaptureCategory category);
+
+            PHONON_DEPRECATED void setAudioCaptureDevice(Phonon::Category category);
 
             /**
              * Sets the video capture source to use.
              *
-             * @param source An object of class VideoCaptureDevice. A list of
-             * available objects can be queried from
-             * BackendCapabilities::availableVideoCaptureDevices.
+             * @param source An object of class VideoCaptureDevice.
              *
              * @see videoCaptureDevice
              * @see setVideoCaptureDevice(int)
              */
             void setVideoCaptureDevice(const Phonon::VideoCaptureDevice &source);
-            void setVideoCaptureDevice(Phonon::Category category);
 
             /**
-             * @deprecated since 4.4.3, use 
+             * Sets the audio capture device by using a capture category to get
+             * the appropriate device.
+             *
+             * @param category Capture category to use for getting a device
+             */
+            void setVideoCaptureDevice(Phonon::CaptureCategory category);
+
+            PHONON_DEPRECATED void setVideoCaptureDevice(Phonon::Category category);
+
+            /**
+             * @deprecated since 4.4.3, use
              * setVideoCaptureDevice(const Phonon::VideoCaptureDevice &source) instead
              */
-            Q_DECL_DEPRECATED void setVideoCaptureDevice(const Phonon::Experimental::VideoCaptureDevice &source);
+            PHONON_DEPRECATED void setVideoCaptureDevice(const Phonon::Experimental::VideoCaptureDevice &source);
 
         public Q_SLOTS:
             /**
@@ -123,10 +161,26 @@ namespace Experimental
              */
             void start();
 
+            /**
+             * Pause capture.
+             */
+            void pause();
+
             /**
              * Stop capture.
              */
             void stop();
+
+        Q_SIGNALS:
+            /**
+             * Emitted when the state of the video or audio capture device has been changed.
+             *
+             * \li If only the audio capture device is valid, it notifies about the audio capture state.
+             * \li If only the video capture device is valid, it notifies about the video capture state.
+             * \li If both the audio and video capture devices are valid, it only notifies about the
+             * video capture state.
+             */
+            void stateChanged(Phonon::State newstate, Phonon::State oldstate);
     };
 
 } // namespace Experimental
