@@ -83,9 +83,9 @@ void AVCaptureTest::testAudioOutput()
 
 void AVCaptureTest::testAudioCapture()
 {
-    #ifndef QT_NO_PHONON_AUDIOCAPTURE
+    #ifndef PHONON_NO_AUDIOCAPTURE
     // Write device indices
-    QList<int> acList = m_pgc->audioCaptureDeviceListFor(Phonon::NoCategory);
+    QList<int> acList = m_pgc->audioCaptureDeviceListFor(Phonon::NoCaptureCategory);
     qDebug() << "Device list for audio capture" << acList;
 
     // Write device properties
@@ -106,9 +106,9 @@ void AVCaptureTest::testAudioCapture()
 
 void AVCaptureTest::testVideoCapture()
 {
-    #ifndef QT_NO_PHONON_VIDEOCAPTURE
+    #ifndef PHONON_NO_VIDEOCAPTURE
     // Write device indices
-    QList<int> acList = m_pgc->videoCaptureDeviceListFor(Phonon::NoCategory);
+    QList<int> acList = m_pgc->videoCaptureDeviceListFor(Phonon::NoCaptureCategory);
     qDebug() << "Device list for video capture" << acList;
 
     // Write device properties
