@@ -1,12 +1,13 @@
-/*  This file is part of the KDE project
+/*
     Copyright (C) 2007 Matthias Kretz <kretz@kde.org>
+    Copyright (C) 2011 Harald Sitter <sitter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) version 3, or any
     later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), Nokia Corporation 
+    successor approved by the membership of KDE e.V.), Nokia Corporation
     (or its successors, if any) and the KDE Free Qt Foundation, which shall
     act as a proxy defined in Section 6 of version 3 of the license.
 
@@ -15,7 +16,7 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public 
+    You should have received a copy of the GNU Lesser General Public
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 */
@@ -53,10 +54,26 @@ class PHONON_EXPORT MediaController : public QObject
         enum Feature {
             Angles = 1,
             Chapters = 2,
+            Navigations = 3,
             Titles = 4
         };
         Q_DECLARE_FLAGS(Features, Feature)
 
+        enum NavigationMenu {
+            RootMenu,     /** < Root/main menu. */
+            TitleMenu,    /** < Title Menu to access different titles on the media source.
+                                The title menu is usually where one would select
+                                the episode of a TV series DVD. It can be equal to
+                                the main menu but does not need to be. */
+            AudioMenu,    /** < Audio menu for language (and somtimes also subtitle)
+                                settings etc. */
+            SubtitleMenu, /** < Subtitle menu. Usually this represents the same menu
+                                as AudioMenu or is not present at all (in which case
+                                subtitle settings are propably also in the AudioMenu). */
+            ChapterMenu,  /** < Chapter menu for chapter selection. */
+            AngleMenu     /** < Angle menu. Rarely supported on any media source. */
+        };
+
         MediaController(MediaObject *parent);
         ~MediaController();
 
@@ -68,6 +85,38 @@ class PHONON_EXPORT MediaController : public QObject
         int availableChapters() const;
         int currentChapter() const;
 
+        /**
+         * Translates a NavigationMenu enum to a string you can use in your GUI.
+         * Please note that keyboard shortucts will not be present in the returned
+         * String, therefore it is probably not a good idea to use this function
+         * if you are providing keyboard shortcuts for every other clickable.
+         *
+         * Please note that RootMenu has the string representation "Main Menu" as
+         * root is a rather technical term when talking about menus.
+         *
+         * Example:
+         * \code
+         * QString s = Phonon::MediaController::navigationMenuToString(MenuMain);
+         * // s now contains "Main Menu"
+         * \endcode
+         *
+         * \returns the QString representation of the menu
+         */
+        static QString navigationMenuToString(NavigationMenu menu);
+
+        /**
+         * Get the list of currently available menus for the present media source.
+         *
+         * The list is always ordered by occurance in the NavgiationMenu enum.
+         * Should you wish to use a different order in your application you will
+         * have to make appropriate changes.
+         *
+         * \returns list of available menus (supported by backend and media source).
+         *
+         * \see navigationMenuToString()
+         */
+        QList<NavigationMenu> availableMenus() const;
+
         int availableTitles() const;
         int currentTitle() const;
 
@@ -96,7 +145,7 @@ class PHONON_EXPORT MediaController : public QObject
          * \see selectedAudioChannel
          * \see setCurrentAudioChannel
          */
-        QList<AudioChannelDescription> availableAudioChannels() const;
+        QList<Phonon::AudioChannelDescription> availableAudioChannels() const;
 
         /**
          * Returns the subtitle streams that can be selected by the user. The
@@ -120,6 +169,13 @@ class PHONON_EXPORT MediaController : public QObject
          */
         void setCurrentAudioChannel(const Phonon::AudioChannelDescription &stream);
 
+        /**
+         * Switches to a menu (e.g. on a DVD).
+         *
+         * \see availableMenus()
+         */
+        void setCurrentMenu(NavigationMenu menu);
+
         /**
          * Selects a subtitle stream from the media.
          *
@@ -163,13 +219,26 @@ class PHONON_EXPORT MediaController : public QObject
         void previousTitle();
 
     Q_SIGNALS:
-        void availableSubtitlesChanged();
-        void availableAudioChannelsChanged();
         void availableAnglesChanged(int availableAngles);
-        void angleChanged(int angleNumber);
+        void availableAudioChannelsChanged();
         void availableChaptersChanged(int availableChapters);
-        void chapterChanged(int chapterNumber);
+
+        /**
+         * The available menus changed, this for example emitted when Phonon switches
+         * from a media source without menus to one with menus (e.g. a DVD).
+         *
+         * \param menus is a list of all currently available menus, you should update
+         * GUI representations of the available menus with the new set.
+         *
+         * \see availableMenus()
+         * \see navigationMenuToString()
+         */
+        void availableMenusChanged(QList<NavigationMenu> menus);
+        void availableSubtitlesChanged();
         void availableTitlesChanged(int availableTitles);
+
+        void angleChanged(int angleNumber);
+        void chapterChanged(int chapterNumber);
         void titleChanged(int titleNumber);
 
     protected:
@@ -180,6 +249,9 @@ class PHONON_EXPORT MediaController : public QObject
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Phonon::MediaController::Features)
 
+Q_DECLARE_METATYPE(Phonon::MediaController::NavigationMenu)
+Q_DECLARE_METATYPE(QList<Phonon::MediaController::NavigationMenu>)
+
 #endif //QT_NO_PHONON_MEDIACONTROLLER
 
 QT_END_NAMESPACE
