@@ -1,5 +1,6 @@
 /*  This file is part of the KDE project
     Copyright (C) 2005-2007 Matthias Kretz <kretz@kde.org>
+    Copyright (C) 2011 Trever Fischer <tdfischer@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -30,9 +31,19 @@
 #include "abstractmediastream_p.h"
 #include "frontendinterface_p.h"
 
+#include <QtCore/QStringBuilder>
 #include <QtCore/QStringList>
-#include <QtCore/QUrl>
+#include <QtCore/QDateTime>
 #include <QtCore/QTimer>
+#include <QtCore/QUrl>
+
+#ifdef HAVE_QZEITGEIST
+#include <QtZeitgeist/DataModel/Event>
+#include <QtZeitgeist/Interpretation>
+#include <QtZeitgeist/Log>
+#include <QtZeitgeist/Manifestation>
+#include <QtZeitgeist/QtZeitgeist>
+#endif
 
 #include "phononnamespace_p.h"
 #include "platform_p.h"
@@ -159,15 +170,15 @@ QStringList MediaObject::metaData(Phonon::MetaData f) const
 {
     switch (f) {
     case ArtistMetaData:
-        return metaData(QLatin1String("ARTIST"     ));
+        return metaData(QLatin1String("ARTIST"));
     case AlbumMetaData:
-        return metaData(QLatin1String("ALBUM"      ));
+        return metaData(QLatin1String("ALBUM"));
     case TitleMetaData:
-        return metaData(QLatin1String("TITLE"      ));
+        return metaData(QLatin1String("TITLE"));
     case DateMetaData:
-        return metaData(QLatin1String("DATE"       ));
+        return metaData(QLatin1String("DATE"));
     case GenreMetaData:
-        return metaData(QLatin1String("GENRE"      ));
+        return metaData(QLatin1String("GENRE"));
     case TracknumberMetaData:
         return metaData(QLatin1String("TRACKNUMBER"));
     case DescriptionMetaData:
@@ -238,19 +249,9 @@ void MediaObject::setCurrentSource(const MediaSource &newSource)
             // for setting a new URL
 
     d->mediaSource = newSource;
-#ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
-    d->kiofallback = 0; // kiofallback auto-deletes
-#endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
-
-//X         if (url.scheme() == "http") {
-//X             d->kiofallback = Platform::createMediaStream(url, this);
-//X             if (d->kiofallback) {
-//X                 ...
-//X                 return;
-//X             }
-//X         }
 
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
+    d->abstractStream = 0; // abstractStream auto-deletes
     if (d->mediaSource.type() == MediaSource::Stream) {
         Q_ASSERT(d->mediaSource.stream());
         d->mediaSource.stream()->d_func()->setMediaObjectPrivate(d);
@@ -318,6 +319,147 @@ void MediaObject::clearQueue()
     d->sourceQueue.clear();
 }
 
+void MediaObjectPrivate::sendToZeitgeist(const QString &event_interpretation,
+                                         const QString &event_manifestation,
+                                         const QString &event_actor,
+                                         const QDateTime &subject_timestamp,
+                                         const QUrl &subject_uri,
+                                         const QString &subject_text,
+                                         const QString &subject_interpretation,
+                                         const QString &subject_manifestation,
+                                         const QString &subject_mimetype)
+{
+#ifdef HAVE_QZEITGEIST
+    QtZeitgeist::DataModel::Subject subject;
+    QString url = subject_uri.toString();
+    QString path = url.left(url.lastIndexOf(QLatin1Char('/')));
+    subject.setUri(url);
+    subject.setText(subject_text);
+    subject.setInterpretation(subject_interpretation);
+    subject.setManifestation(subject_manifestation);
+    subject.setOrigin(path);
+    subject.setMimeType(subject_mimetype);
+
+    QtZeitgeist::DataModel::SubjectList subjects;
+    subjects << subject;
+
+    QtZeitgeist::DataModel::Event event;
+    event.setTimestamp(subject_timestamp);
+    event.setInterpretation(event_interpretation);
+    event.setManifestation(event_manifestation);
+    event.setActor(event_actor);
+    event.setSubjects(subjects);
+
+    QtZeitgeist::DataModel::EventList events;
+    events << event;
+
+    QDBusPendingReply<QtZeitgeist::DataModel::EventIdList> reply =
+        log->insertEvents(events);
+#else
+    Q_UNUSED(event_interpretation)
+    Q_UNUSED(event_manifestation)
+    Q_UNUSED(event_actor)
+    Q_UNUSED(subject_timestamp)
+    Q_UNUSED(subject_uri)
+    Q_UNUSED(subject_text)
+    Q_UNUSED(subject_interpretation)
+    Q_UNUSED(subject_manifestation)
+    Q_UNUSED(subject_mimetype)
+#endif
+}
+
+void MediaObjectPrivate::sendToZeitgeist(State eventState)
+{
+#ifdef HAVE_QZEITGEIST
+    Q_Q(MediaObject);
+    if (readyForZeitgeist && q->property("PlaybackTracking").toBool()) {
+        pDebug() << "Current state:" << eventState;
+        QString eventInterpretation;
+        switch (eventState) {
+        case PlayingState:
+            eventInterpretation = QtZeitgeist::Interpretation::Event::ZGAccessEvent;
+            break;
+        case ErrorState:
+        case StoppedState:
+            eventInterpretation = QtZeitgeist::Interpretation::Event::ZGLeaveEvent;
+            break;
+        //These states are not signifigant events.
+        case LoadingState:
+        case BufferingState:
+        case PausedState:
+            return;
+            break;
+        }
+
+        QStringList titles = q->metaData(TitleMetaData);
+        QStringList artists = q->metaData(ArtistMetaData);
+        QString title;
+        if (titles.empty()) {
+            QString file = mediaSource.url().toString();
+            title = file.right(file.length()-file.lastIndexOf("/")-1);
+        } else {
+            if (artists.empty()) {
+                title = titles[0];
+            } else {
+                title = QString(QObject::tr("%0 by %1")).arg(titles[0]).arg(artists[0]);
+            }
+        }
+        pDebug() << "Sending" << title << "to zeitgeist";
+
+        QString mime;
+        QString subjectInterpretation;
+        if (q->hasVideo()) {
+            subjectInterpretation = QtZeitgeist::Interpretation::Subject::NFOVideo;
+            mime = "video/raw";
+        } else {
+            subjectInterpretation = QtZeitgeist::Interpretation::Subject::NFOAudio;
+            mime = "audio/raw";
+        }
+        pDebug() << "Zeitgeist mime type:" << mime;
+        pDebug() << "Zeitgeist URL:" << mediaSource.url();
+        pDebug() << "mediasource type:" << mediaSource.type();
+
+        QString subjectType;
+        switch (mediaSource.type()) {
+        case MediaSource::Empty:
+        case MediaSource::Invalid:
+            return;
+        case MediaSource::Url:
+            subjectType = QtZeitgeist::Manifestation::Subject::NFORemoteDataObject;
+            break;
+        case MediaSource::CaptureDevice:
+        case MediaSource::Disc:
+        case MediaSource::Stream:
+            subjectType = QtZeitgeist::Manifestation::Subject::NFOMediaStream;
+            break;
+        case MediaSource::LocalFile:
+            subjectType = QtZeitgeist::Manifestation::Subject::NFOFileDataObject;
+            break;
+        }
+
+        sendToZeitgeist(eventInterpretation,
+                        QtZeitgeist::Manifestation::Event::ZGUserActivity,
+                        QLatin1Literal("app://" ) % Platform::applicationName() % QLatin1Literal(".desktop"),
+                        QDateTime::currentDateTime(),
+                        mediaSource.url(),
+                        title,
+                        subjectInterpretation,
+                        subjectType,
+                        mime);
+    }
+    // Unset this so we don't send it again after a pause+play
+    readyForZeitgeist = false;
+#else
+    Q_UNUSED(eventState)
+#endif
+}
+
+void MediaObjectPrivate::sendToZeitgeist()
+{
+    Q_Q(MediaObject);
+    sendToZeitgeist(q->state());
+}
+
 bool MediaObjectPrivate::aboutToDeleteBackendObject()
 {
     //pDebug() << Q_FUNC_INFO;
@@ -344,11 +486,21 @@ void MediaObjectPrivate::streamError(Phonon::ErrorType type, const QString &text
     QMetaObject::invokeMethod(q, "stateChanged", Qt::QueuedConnection, Q_ARG(Phonon::State, Phonon::ErrorState), Q_ARG(Phonon::State, lastState));
     //emit q->stateChanged(ErrorState, lastState);
 }
+#endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
 
+// TODO: this needs serious cleanup...
 void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State oldstate)
 {
     Q_Q(MediaObject);
 
+    // Zeitgeist ---------------------------------------------------------------
+    if (newstate == StoppedState) {
+        readyForZeitgeist = true;
+    }
+    pDebug() << "State changed from" << oldstate << "to" << newstate << "-> sending to zeitgeist.";
+    sendToZeitgeist(newstate);
+
+    // AbstractMediaStream fallback stuff --------------------------------------
     if (errorOverride) {
         errorOverride = false;
         if (newstate == ErrorState) {
@@ -364,9 +516,9 @@ void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State o
     }
 
     // backend MediaObject reached ErrorState, try a KioMediaSource
-    if (newstate == Phonon::ErrorState && !kiofallback) {
-        kiofallback = Platform::createMediaStream(mediaSource.url(), q);
-        if (!kiofallback) {
+    if (newstate == Phonon::ErrorState && !abstractStream) {
+        abstractStream = Platform::createMediaStream(mediaSource.url(), q);
+        if (!abstractStream) {
             pDebug() << "backend MediaObject reached ErrorState, no KIO fallback available";
             emit q->stateChanged(newstate, oldstate);
             return;
@@ -390,8 +542,8 @@ void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State o
             emit q->stateChanged(Phonon::LoadingState, oldstate);
             break;
         }
-        kiofallback->d_func()->setMediaObjectPrivate(this);
-        MediaSource mediaSource(kiofallback);
+        abstractStream->d_func()->setMediaObjectPrivate(this);
+        MediaSource mediaSource(abstractStream);
         mediaSource.setAutoDelete(true);
         pINTERFACE_CALL(setSource(mediaSource));
         if (oldstate == Phonon::BufferingState) {
@@ -399,13 +551,13 @@ void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State o
         }
         return;
     } else if (ignoreLoadingToBufferingStateChange &&
-            kiofallback &&
+            abstractStream &&
             oldstate == Phonon::LoadingState) {
         if (newstate != Phonon::BufferingState) {
             emit q->stateChanged(newstate, Phonon::BufferingState);
         }
         return;
-    } else if (ignoreErrorToLoadingStateChange && kiofallback && oldstate == ErrorState) {
+    } else if (ignoreErrorToLoadingStateChange && abstractStream && oldstate == ErrorState) {
         if (newstate != LoadingState) {
             emit q->stateChanged(newstate, Phonon::LoadingState);
         }
@@ -414,7 +566,6 @@ void MediaObjectPrivate::_k_stateChanged(Phonon::State newstate, Phonon::State o
 
     emit q->stateChanged(newstate, oldstate);
 }
-#endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
 
 void MediaObjectPrivate::_k_aboutToFinish()
 {
@@ -422,7 +573,7 @@ void MediaObjectPrivate::_k_aboutToFinish()
     pDebug() << Q_FUNC_INFO;
 
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
-    kiofallback = 0; // kiofallback auto-deletes
+    abstractStream = 0; // abstractStream auto-deletes
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
 
     if (sourceQueue.isEmpty()) {
@@ -433,6 +584,7 @@ void MediaObjectPrivate::_k_aboutToFinish()
     }
 
     mediaSource = sourceQueue.head();
+    readyForZeitgeist = false;
     pINTERFACE_CALL(setNextSource(mediaSource));
 }
 
@@ -516,6 +668,7 @@ void MediaObjectPrivate::setupBackendObject()
 
     // set up attributes
     if (isPlayable(mediaSource.type())) {
+        readyForZeitgeist = false;
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
         if (mediaSource.type() == MediaSource::Stream) {
             Q_ASSERT(mediaSource.stream());
@@ -546,6 +699,9 @@ void MediaObjectPrivate::_k_metaDataChanged(const QMultiMap<QString, QString> &n
 {
     metaData = newMetaData;
     emit q_func()->metaDataChanged();
+    pDebug() << "Metadata ready, sending to zeitgeist";
+    readyForZeitgeist = true;
+    sendToZeitgeist();
 }
 
 void MediaObjectPrivate::phononObjectDestroyed(MediaNodePrivate *bp)
