@@ -6,7 +6,7 @@
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) version 3, or any
     later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), Trolltech ASA 
+    successor approved by the membership of KDE e.V.), Nokia Corporation
     (or its successors, if any) and the KDE Free Qt Foundation, which shall
     act as a proxy defined in Section 6 of version 3 of the license.
 
@@ -15,7 +15,7 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public 
+    You should have received a copy of the GNU Lesser General Public
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 */
@@ -29,6 +29,10 @@
 #include "medianodedestructionhandler_p.h"
 #include "mediasource.h"
 #include <QtCore/QQueue>
+#ifdef HAVE_QZEITGEIST
+#include <QtZeitgeist/Log>
+#include <QtZeitgeist/QtZeitgeist>
+#endif
 
 QT_BEGIN_NAMESPACE
 
@@ -46,7 +50,34 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
     Q_DECLARE_PUBLIC(MediaObject)
     public:
         virtual QObject *qObject() { return q_func(); }
-        QList<FrontendInterfacePrivate *> interfaceList;
+
+        /**
+         * Sends the metadata for this media file to the Zeitgeist tracker
+         *
+         * \param eventInterpretation The interpretation of the event
+         * \param eventManifestation The manifestation type of the event
+         * \param eventActor The application or entity responsible for emitting the zeitgeist event
+         * \param eventTimestamp The time
+         * \param subjectURI The file's URI
+         * \param subjectText A free-form annotation
+         * \param subjectInterpretation The interpretation type
+         * \param subjectManifestation The manifestation type
+         * \param subjectMimetype The file's mimetype
+         */
+        void sendToZeitgeist(const QString &event_interpretation,
+                             const QString &event_manifestation,
+                             const QString &event_actor,
+                             const QDateTime &subject_timestamp,
+                             const QUrl &subject_uri,
+                             const QString &subject_text,
+                             const QString &subject_interpretation,
+                             const QString &subject_manifestation,
+                             const QString &subject_mimetype);
+
+        void sendToZeitgeist(State);
+        void sendToZeitgeist();
+
+    QList<FrontendInterfacePrivate *> interfaceList;
     protected:
         virtual bool aboutToDeleteBackendObject();
         virtual void createBackendObject();
@@ -58,9 +89,9 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
         void _k_metaDataChanged(const QMultiMap<QString, QString> &);
         void _k_aboutToFinish();
         void _k_currentSourceChanged(const MediaSource &);
+        PHONON_EXPORT void _k_stateChanged(Phonon::State, Phonon::State);
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
         void streamError(Phonon::ErrorType, const QString &);
-        PHONON_EXPORT void _k_stateChanged(Phonon::State, Phonon::State);
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
 
         MediaObjectPrivate()
@@ -71,9 +102,10 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
             prefinishMark(0),
             transitionTime(0), // gapless playback
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
-            kiofallback(0),
+            abstractStream(0),
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
-            state(Phonon::LoadingState)
+            state(Phonon::LoadingState),
+            readyForZeitgeist(false)
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
             , errorType(Phonon::NormalError),
             errorOverride(false),
@@ -81,6 +113,10 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
             ignoreErrorToLoadingStateChange(false)
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
         {
+#ifdef HAVE_QZEITGEIST
+            QtZeitgeist::init();
+            log = new QtZeitgeist::Log(qObject());
+#endif
         }
 
         qint64 currentTime;
@@ -90,13 +126,14 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
         qint32 prefinishMark;
         qint32 transitionTime;
 #ifndef QT_NO_PHONON_ABSTRACTMEDIASTREAM
-        AbstractMediaStream *kiofallback;
+        AbstractMediaStream *abstractStream;
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
         State state
 #ifdef QT_NO_PHONON_ABSTRACTMEDIASTREAM
             ;
 #else
             : 8;
+        bool readyForZeitgeist;
         ErrorType errorType : 4;
         bool errorOverride : 1;
         bool ignoreLoadingToBufferingStateChange : 1;
@@ -104,6 +141,9 @@ class MediaObjectPrivate : public MediaNodePrivate, private MediaNodeDestruction
 #endif //QT_NO_PHONON_ABSTRACTMEDIASTREAM
         MediaSource mediaSource;
         QQueue<MediaSource> sourceQueue;
+#ifdef HAVE_QZEITGEIST
+        QtZeitgeist::Log *log;
+#endif
 };
 }
 
